#ifndef MPIPE_API_COMMAND_H
#define MPIPE_API_COMMAND_H

#include "mpipe_types.h"
#include "mpipe_common.h"

// описание иконки курсора
typedef struct mpipe_cursor_image_t
{
    enum mpipe_video_format_id_t            image_format_id;// ID-формата иконки
    mpipe_rect_t                            image_rect;     // размер иконки
    struct mpipe_data_t                     image_data;     // данные иконки
} mpipe_cursor_image_t;


// параметры команды для обмена SDP
typedef struct mpipe_command_sdp_params_t
{
    enum mpipe_peer_type_t                  peer_type;      // тип пира: local, remote
    enum mpipe_sdp_type_t                   sdp_type;       // тип SDP: offe, answer
    const char*                             sdp;            // строка в формате SDP

} mpipe_command_sdp_params_t;

typedef struct mpipe_data_t mpipe_command_data_params_t;
/*// параметры команды для пользовательских данных
typedef struct mpipe_command_data_params_t
{
    const void*                             data;           // указатель на данные
    uint32_t                                size;           // размер данных

} mpipe_command_data_params_t;*/

// параметры команды перемещения курсором
typedef struct mpipe_command_move_cursor_params_t
{
    mpipe_position_t                        position;       // новая позиция курсора для пермещения

} mpipe_command_move_cursor_params_t;

// параметры команды нажатия клавиш (коды клавиш будут уточняться)
typedef struct mpipe_command_press_keys_params_t
{
    const uint32_t*                         keys;           // указатель на массив клавиш
    uint32_t                                count;          // количество клавиш заданных в массиве

} mpipe_command_press_keys_params_t;


// параметры команды нажатия клавиш (коды клавиш будут уточняться)
typedef struct mpipe_command_cursor_state_params_t
{
    mpipe_position_t                        position;       // позиция курсора
    mpipe_cursor_image_t                    image;          // иконка курсора
} mpipe_command_cursor_state_params_t;


// параметры команды передачи буфера обмена
typedef struct mpipe_command_clipboard_params_t
{
    mpipe_mime_type_t                       mime_type;      // тип данных (поддерживается только text_plain)
    struct mpipe_data_t                     data;           // данные буфера обмена
} mpipe_command_clipboard_params_t;

// общая структура команды
typedef struct mpipe_command_t
{
    enum mpipe_command_id_t                 command_id;     // ID-команды: sdp, data
    mpipe_timestamp_t                       timestamp;      // метка времени в наносекундныйх тиках от старта системы
    union
    {
        mpipe_command_sdp_params_t          sdp;            // Параметры для SDP
        mpipe_command_data_params_t         data;           // Параметры пользовательских данных
        mpipe_command_move_cursor_params_t  move_cursor;    // Параметры перемещеняи курсора
        mpipe_command_press_keys_params_t   press_keys;     // Параметры нажатия клавиш
        mpipe_command_cursor_state_params_t cursor_state;   // Параметры состояния курсора
        mpipe_command_clipboard_params_t    clipboard;      // Параметры буфера обмена
    }                                       params;

} mpipe_command_t;



#endif // MPIPE_API_COMMAND_H
