#ifndef MPIPE_IMAGE_IMPL_H
#define MPIPE_IMAGE_IMPL_H

#include "video_info.h"
#include "i_image.h"

namespace mpipe
{

class image_impl_basic : public i_image
{
protected:
    video_info_t                m_info;
    video_info_t::frame_point_t m_offset;

public:
    image_impl_basic(const video_info_t& info = {}
                     , const video_info_t::frame_point_t& offset = {});

    void set_info(const video_info_t& info);
    void set_info(video_info_t&& info);
    void set_offset(const video_info_t::frame_point_t& offset);

    video_info_t::frame_point_t& offset();
    const video_info_t::frame_point_t& offset() const;

    video_info_t& info();
    const video_info_t& info() const;

    // i_image interface
public:
    int32_t hot_x() const override;
    int32_t hot_y() const override;
    int32_t width() const override;
    int32_t height() const override;
    video_format_id_t format() const override;
};

template<class Buffer>
class image_impl : public image_impl_basic
{
    Buffer      m_image_buffer;

public:
    using u_ptr_t = std::unique_ptr<image_impl<Buffer>>;
    using s_ptr_t = std::shared_ptr<image_impl<Buffer>>;

    static u_ptr_t create(const video_info_t& info
                          , const Buffer& buffer = {}
                          , const video_info_t::frame_point_t& offset = {});
    static u_ptr_t create(const video_info_t& info = {}
                          , Buffer&& buffer = {}
                         , const video_info_t::frame_point_t& offset= {});

    image_impl(const video_info_t& info
               , const Buffer& buffer= {}
               , const video_info_t::frame_point_t& offset = {});
    image_impl(const video_info_t& info = {}
              , Buffer&& buffer = {}
              , const video_info_t::frame_point_t& offset = {});

    void set_image_data(const Buffer& buffer);
    void set_image_data(Buffer&& buffer);

    Buffer& image_buffer();
    const Buffer& image_buffer() const;

    // i_image interface
public:
    const i_data_object &data() const override;
    i_image::u_ptr_t clone() const override;
};

}

#endif // MPIPE_IMAGE_IMPL_H
