#ifndef MPIPE_MEDIA_VARIANT_INFO_H
#define MPIPE_MEDIA_VARIANT_INFO_H

#include "audio_info.h"
#include "video_info.h"
#include "undefined_info.h"

#include <variant>

namespace mpipe
{

class i_option;
class i_media_format;

struct variant_info_t
{
    using variant_t = std::variant<undefined_info_t, audio_info_t, video_info_t>;

    variant_t       info;

    variant_info_t();

    variant_info_t(const i_media_format& media_format);

    template<typename T>
    variant_info_t(const T& variant_value);

    media_type_t media_type() const;

    template<typename T>
    variant_info_t& assign(const T& variant_value);

    template<typename T>
    variant_info_t& assign(T&& variant_value);

    variant_info_t& assign(const i_media_format& media_format);

    template<typename T>
    T& get_info();

    template<typename T>
    const T& get_info() const;

    bool operator == (const variant_info_t& other) const;
    bool operator != (const variant_info_t& other) const;

    bool is_valid() const;

    bool serialize(i_option& option) const;
    bool deserialize(const i_option& option);


};

}

#endif // MPIPE_MEDIA_VARIANT_INFO_H
