#ifndef PORTABLE_SPC_SPACE_REQUEST_H
#define PORTABLE_SPC_SPACE_REQUEST_H

#include "space_types.h"
#include "cursor_state.h"
#include "clipboard_state.h"
#include <vector>
#include <variant>

namespace pt::spc
{

struct space_request_t
{
    using array_t = std::vector<space_request_t>;
    using keys_array_t = std::vector<key_code_t>;

    using request_params_t = std::variant<cursor_pos_t, keys_array_t, clipboard_state_t>;

    request_type_t          request_type;
    std::size_t             request_id;
    request_params_t        params;


    space_request_t(request_type_t request_type = request_type_t::none
                    , std::size_t request_id = 0);
    space_request_t(const cursor_pos_t& move_cursor
                    , std::size_t request_id = 0);
    space_request_t(const keys_array_t& press_keys
                    , std::size_t request_id = 0);

    space_request_t(const clipboard_state_t& clipboard_state
                    , std::size_t request_id = 0);

    clipboard_state_t& clipboard_state();
    const clipboard_state_t& clipboard_state() const;

    cursor_pos_t& move_cursor();
    const cursor_pos_t& move_cursor() const;

    keys_array_t& press_keys();
    const keys_array_t& press_keys() const;

    bool is_valid() const;

};

}

#endif // PORTABLE_SPC_SPACE_REQUEST_H
