#ifndef PORTABLE_SPC_SPACE_RESPONSE_H
#define PORTABLE_SPC_SPACE_RESPONSE_H

#include "space_state.h"
#include "space_types.h"
#include "clipboard_state.h"
#include <variant>

namespace pt::spc
{

struct space_response_t
{
    using response_params_t = std::variant<cursor_state_t, image_view_t, space_state_t, clipboard_state_t>;

    request_type_t          request_type;
    std::size_t             reqest_id;
    result_t                result;
    response_params_t       params;

    space_response_t(request_type_t request_type = request_type_t::none
                     , std::size_t reqest_id = 0
                     , result_t result = result_t::undefined);

    space_response_t(const cursor_state_t& cursor_state
                     , std::size_t reqest_id = 0
                     , result_t result = result_t::undefined);

    space_response_t(const image_view_t& screen_image
                     , std::size_t reqest_id = 0
                     , result_t result = result_t::undefined);

    space_response_t(const space_state_t& space_state
                     , std::size_t reqest_id = 0
                     , result_t result = result_t::undefined);

    space_response_t(const clipboard_state_t& clipboard_state
                     , std::size_t reqest_id = 0
                     , result_t result = result_t::undefined);


    cursor_state_t& cursor_state();
    const cursor_state_t& cursor_state() const;

    image_view_t& screen_image();
    const image_view_t& screen_image() const;

    space_state_t& space_state();
    const space_state_t& space_state() const;

    clipboard_state_t& clipboard_state();
    const clipboard_state_t& clipboard_state() const;

    bool is_valid() const;
    bool has_params() const;
    bool is_success() const;
};

}

#endif // PORTABLE_SPC_SPACE_RESPONSE_H
