#ifndef PORTABLE_UTILS_DATA_FRAME_H
#define PORTABLE_UTILS_DATA_FRAME_H

#include <vector>
#include "data_view.h"

namespace pt::utils
{

struct data_frame
{
    using raw_array_t = std::vector<std::uint8_t>;
    data_view_t     m_data_view;
    raw_array_t     m_stored_array;

public:
    static raw_array_t create_array(const void* data
                                    , std::size_t size);

    data_frame(const void* data = nullptr
               , std::size_t size = 0);

    data_frame(const data_view_t& data_view);

    data_frame(raw_array_t&& array);


    void assign(const void* data
                , std::size_t size);

    void assign(const data_view_t& data_view);

    void assign(raw_array_t&& array);

    const void* data() const;
    std::size_t size() const;
    void* map();
    void resize(std::size_t new_size);

    void make_stored();
    void reset();

    bool is_stored() const;
    bool is_valid() const;
    bool is_empty() const;

    bool operator == (const data_frame& other) const;
    bool operator != (const data_frame& other) const;

};

}

#endif // PORTABLE_UTILS_DATA_FRAME_H
